import numpy as np
import tlc
from ultralytics.models.yolo.segment.val import SegmentationValidator
from ultralytics.utils import ops

from tlc_ultralytics.constants import (
    IMAGE_COLUMN_NAME,
    SEGMENTATION_LABEL_COLUMN_NAME,
)
from tlc_ultralytics.detect.validator import TLCDetectionValidator
from tlc_ultralytics.utils.dataset import check_tlc_dataset


class TLCSegmentationValidator(TLCDetectionValidator, SegmentationValidator):
    _default_image_column_name = IMAGE_COLUMN_NAME
    _default_label_column_name = SEGMENTATION_LABEL_COLUMN_NAME

    def check_dataset(self, *args, **kwargs):
        return check_tlc_dataset(*args, task="segment", settings=self._settings, **kwargs)

    def init_metrics(self, model):
        """Initialize metrics and use native mask processing for full-size masks."""
        super().init_metrics(model)
        # Always use native mask processing for 3LC to get full-size masks
        self.process = ops.process_mask_native

    def _get_metrics_schemas(self) -> dict[str, tlc.Schema]:
        # TODO: Ensure class  mapping is the same as in input table
        instance_properties_structure = {
            tlc.LABEL: tlc.CategoricalLabel(name=tlc.LABEL, classes=self.data["names_3lc"]),
            tlc.CONFIDENCE: tlc.Float(name=tlc.CONFIDENCE, number_role=tlc.NUMBER_ROLE_CONFIDENCE),
        }

        segment_sample_type = tlc.InstanceSegmentationMasks(
            name=tlc.PREDICTED_SEGMENTATIONS,
            instance_properties_structure=instance_properties_structure,
            is_prediction=True,
        )

        return {tlc.PREDICTED_SEGMENTATIONS: segment_sample_type.schema}

    def _compute_3lc_metrics(self, preds, batch):
        return {tlc.PREDICTED_SEGMENTATIONS: self._process_predictions(preds, batch)}

    def _build_annotation(self, scaled, mapped_classes, h, w):
        masks = scaled["masks"].cpu().numpy()
        masks = np.transpose(masks, (1, 2, 0))  # (N, H, W) -> (H, W, N)
        return {
            tlc.IMAGE_HEIGHT: h,
            tlc.IMAGE_WIDTH: w,
            tlc.INSTANCE_PROPERTIES: {
                tlc.LABEL: mapped_classes,
                tlc.CONFIDENCE: scaled["conf"].tolist(),
            },
            tlc.MASKS: masks,
        }

    def _empty_annotation(self, h, w):
        return {
            tlc.IMAGE_HEIGHT: h,
            tlc.IMAGE_WIDTH: w,
            tlc.INSTANCE_PROPERTIES: {tlc.LABEL: [], tlc.CONFIDENCE: []},
            tlc.MASKS: np.zeros((h, w, 0), dtype=np.uint8),
        }



import numpy as np
import tlc
from tlc.core.builtins.constants import (
    CONFIDENCE,
    INSTANCES,
    INSTANCES_ADDITIONAL_DATA,
    LABEL,
    X_MAX,
    X_MIN,
    Y_MAX,
    Y_MIN,
)
from tlc.core.builtins.schemas import CategoricalLabelListSchema, Float32ListSchema, Geometry2DSchema
from ultralytics.models.yolo.obb.val import OBBValidator

from tlc_ultralytics.constants import (
    IMAGE_COLUMN_NAME,
    OBB_LABEL_COLUMN_NAME,
)
from tlc_ultralytics.detect.validator import TLCDetectionValidator
from tlc_ultralytics.utils.dataset import check_tlc_dataset


class TLCOBBValidator(TLCDetectionValidator, OBBValidator):
    _default_image_column_name = IMAGE_COLUMN_NAME
    _default_label_column_name = OBB_LABEL_COLUMN_NAME

    def check_dataset(self, *args, **kwargs):
        return check_tlc_dataset(*args, task="obb", settings=self._settings, **kwargs)

    def _get_metrics_schemas(self) -> dict[str, tlc.Schema]:
        return {
            "oriented_bbs_2d_predicted": Geometry2DSchema(
                include_2d_oriented_bounding_boxes=True,
                per_instance_schemas={
                    LABEL: CategoricalLabelListSchema(classes=self.data["names"]),
                    CONFIDENCE: Float32ListSchema(),
                },
            )
        }

    def _compute_3lc_metrics(self, preds, batch):
        return {"oriented_bbs_2d_predicted": self._process_predictions(preds, batch)}

    def _build_annotation(self, scaled, mapped_classes, h, w):
        instances = []
        for j in range(len(mapped_classes)):
            bb = scaled["bboxes"][j].cpu().numpy().astype(np.float32).tolist()
            instances.append(
                {
                    "oriented_bbs_2d": [
                        {
                            "center_x": bb[0],
                            "center_y": bb[1],
                            "size_x": bb[2],
                            "size_y": bb[3],
                            "rotation": bb[4],
                        }
                    ],
                }
            )
        return {
            X_MIN: 0,
            Y_MIN: 0,
            X_MAX: w,
            Y_MAX: h,
            INSTANCES: instances,
            INSTANCES_ADDITIONAL_DATA: {
                LABEL: [int(c) for c in mapped_classes],
                CONFIDENCE: scaled["conf"].cpu().numpy().astype(np.float32).tolist(),
            },
        }

    def _empty_annotation(self, h, w):
        return {
            X_MIN: 0,
            Y_MIN: 0,
            X_MAX: w,
            Y_MAX: h,
            INSTANCES: [],
            INSTANCES_ADDITIONAL_DATA: {LABEL: [], CONFIDENCE: []},
        }
